package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.entity.PlantPredictedPowerLongTermDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataMapper;
import pps.core.prediction.service.data.energy_consumption_analysis.GetEnergyConsumptionAnalysisInput;
import pps.core.prediction.service.data.energy_consumption_analysis.GetEnergyConsumptionAnalysisOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 能耗分析模块(测试用)
 * todo: 模拟测试用,后续替换
 *
 * @author ZWT
 * @date 2023/09/27 15:57
 */
@XText("能耗分析模块")
@XService
public class EnergyConsumptionAnalysisService {

    /**
     * 能耗分析--发电列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetEnergyConsumptionAnalysisOutput}>
     */
    @XText("能耗分析--发电列表")
    @XApiAnonymous
    @XApiGet
    public XListResult<GetEnergyConsumptionAnalysisOutput> queryEnergyConsumptionAnalysisList(XContext context, GetEnergyConsumptionAnalysisInput input) {
        //通过线路ID查光伏电站
        List<DynamicQueryBasePowerLinePlantOutput> plantListByLineIds = this.getPowerLinePlantListByLineIds(context, CollUtil.newArrayList(input.getLineId()));
        List powerGenerationList = null;
        List<String> plantIdList = null;
        if (CollUtil.isNotEmpty(plantListByLineIds)) {
            plantIdList = plantListByLineIds.stream()
                    .map(DynamicQueryBasePowerLinePlantOutput::getPlantId)
                    .collect(Collectors.toList());
            //取当前时间
            DateTime date = DateUtil.date();
            PlantPredictedPowerDataMapper powerDataMapper;
            switch (input.getDateType()) {
                case 0:
                    powerDataMapper = context.getBean(PlantPredictedPowerDataMapper.class);
                    powerGenerationList = powerDataMapper.selectList(new QueryWrapper<PlantPredictedPowerDataEnt>()
                            .select("hour_time",
                                    "SUM( power ) AS power")
                            .lambda()
                            .eq(PlantPredictedPowerDataEnt::getYearTime, String.valueOf(date.year()))
                            .eq(PlantPredictedPowerDataEnt::getMonthTime, BaseUtils.getMonthString(date.month() + 1))
                            .eq(PlantPredictedPowerDataEnt::getDayTime, String.valueOf(date.dayOfMonth()))
                            .in(PlantPredictedPowerDataEnt::getPlantId, plantIdList)
                            .groupBy(PlantPredictedPowerDataEnt::getHourTime)
                            .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
                    );
                    break;
                case 1:
                    powerDataMapper = context.getBean(PlantPredictedPowerDataMapper.class);
                    powerGenerationList = powerDataMapper.selectList(new QueryWrapper<PlantPredictedPowerDataEnt>()
                            .select("hour_time",
                                    "SUM( power ) AS power")
                            .lambda()
                            .in(PlantPredictedPowerDataEnt::getPlantId, plantIdList)
                            .between(PlantPredictedPowerDataEnt::getDataDate,
                                    date.toString(BusinessConstant.DATE_FORMAT_DAY),
                                    DateUtil.offsetDay(date, 7).toString(BusinessConstant.DATE_FORMAT_DAY))
                            .groupBy(PlantPredictedPowerDataEnt::getHourTime)
                            .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
                    );
                    break;
                case 2:
                    PlantPredictedPowerLongTermDataMapper longTermDataMapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
                    powerGenerationList = longTermDataMapper.selectList(new QueryWrapper<PlantPredictedPowerLongTermDataEnt>()
                            .select("hour_time",
                                    "SUM( power ) AS power")
                            .lambda()
                            .in(PlantPredictedPowerLongTermDataEnt::getPlantId, plantIdList)
                            .between(PlantPredictedPowerLongTermDataEnt::getDataDate,
                                    date.toString(BusinessConstant.DATE_FORMAT_DAY),
                                    DateUtil.offsetDay(date, 30).toString(BusinessConstant.DATE_FORMAT_DAY))
                            .groupBy(PlantPredictedPowerLongTermDataEnt::getHourTime)
                            .orderByAsc(PlantPredictedPowerLongTermDataEnt::getHourTime)
                    );
                    break;
                default:
            }
        }
        List<GetEnergyConsumptionAnalysisOutput> outputList = new ArrayList<>(0);
        if (CollUtil.isNotEmpty(powerGenerationList)) {
            XCopyUtils.copyList(powerGenerationList, outputList, GetEnergyConsumptionAnalysisOutput.class);
            //有发电量说明有电站ID,查电站详情
            List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPhotovoltaicPlantByPlantIds(context, plantIdList);
            Optional<BigDecimal> reduce = plantList.stream()
                    .map(GetBasePhotovoltaicPlantCloudOutput::getTotalPower)
                    .reduce(BigDecimal::add);
            if (reduce.isPresent()) {
                //负荷率=实际发电量/理论最大发电量x100%
                BigDecimal maximumPowerGeneration = reduce.get();
                BigDecimal power;
                BigDecimal oneHundred = new BigDecimal(100);
                for (GetEnergyConsumptionAnalysisOutput output : outputList) {
                    power = output.getPower();
                    if (Objects.isNull(power) ||
                            power.compareTo(BigDecimal.ZERO) <= BusinessConstant.ZERO ||
                            maximumPowerGeneration.compareTo(BigDecimal.ZERO) == BusinessConstant.ZERO
                    ) {
                        output.setProductionLoad(BigDecimal.ZERO);
                        continue;
                    }
                    //计算负荷
                    output.setProductionLoad(
                            power.divide(maximumPowerGeneration, 4, BigDecimal.ROUND_HALF_UP)
                                    .multiply(oneHundred)
                    );
                }
            }
        }
        return XListResult.success(outputList);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 按线路ID集合获取电站列表
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantOutput}>
     */
    private List<DynamicQueryBasePowerLinePlantOutput> getPowerLinePlantListByLineIds(XContext context, List<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantOutput> result = cloudService.queryPowerLinePlantListByParam(context,
                DynamicQueryBasePowerLinePlantInput.builder()
                        .lineIds(lineIds)
                        .build()
        );
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取光伏发电厂
     *
     * @param context  上下文
     * @param plantIds 植物id
     * @return {@link List}<{@link GetBasePhotovoltaicPlantCloudOutput}>
     */
    private List<GetBasePhotovoltaicPlantCloudOutput> getPhotovoltaicPlantByPlantIds(XContext context, List<String> plantIds) {
        BasePhotovoltaicPlantCloudService cloudService = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = cloudService.getBasePhotovoltaicPlantList(context,
                GetBasePhotovoltaicPlantCloudInput.builder()
                        .plantIds(plantIds)
                        .build());
        result.throwIfFail();
        return result.getResult();
    }
}
